local function on_init()
	global.available_logi_bots = global.available_logi_bots or {}
	global.available_con_bots = global.available_con_bots or {}
	global.charging_bots = global.charging_bots or {}
	global.to_charge_bots = global.to_charge_bots or {}
	global.cycle = global.cycle or 0
	global.update_ticks = settings.player["update-ticks"].value
	global.avg_seconds = settings.player["avg-seconds"].value
end
script.on_init(on_init)

local function zero_averages()
	global.available_logi_bots = {}
	global.available_con_bots = {}
	global.charging_bots = {}
	global.to_charge_bots = {}
	global.cycle = 0
end

local function on_runtime_mod_setting_changed(event)
	local player = game.players[event.player_index]
	local player_settings = settings.get_player_settings(player)
	global.update_ticks = player_settings["update-ticks"].value
	global.avg_seconds = player_settings["avg-seconds"].value
	zero_averages()
end
script.on_event(defines.events.on_runtime_mod_setting_changed, on_runtime_mod_setting_changed)

local function on_configuration_changed(event)
	if event.mod_changes then
		game.print("BotStats: Game or mod version changes detected")
		local changes = event.mod_changes["BotStats"]
		if changes then
			game.print("BotStats: Updated from "..tostring(changes.old_version).." to "..tostring(changes.new_version))
		end
	end
	on_init()
end
script.on_configuration_changed(on_configuration_changed)

local function get_avg(list)
	local sum = 0
	local count = 0
	for _, e in pairs(list) do
		sum = sum + e
		count = count + 1
	end
  return sum / count
end

local function zero_gui(player)
	player.gui.left.bot_stat_gui.available_logi_bots_label.caption = "Logistic: 0/0 of 0"
	player.gui.left.bot_stat_gui.available_con_bots_label.caption = "Construction: 0/0 of 0"
	player.gui.left.bot_stat_gui.charging_bots_label.caption = "Charging: 0/0"
	player.gui.left.bot_stat_gui.to_charge_bots_label.caption = "Waiting: 0/0"
end

local function write_stats(player)
	if not player.gui.left.bot_stat_gui then return end
	local network = player.force.find_logistic_network_by_position(player.position, player.surface)
	if not network then
		zero_gui(player)
		zero_averages()
		return
	end
	local available_logi_bots = network.available_logistic_robots
	local available_con_bots = network.available_construction_robots
	local charging_bots = 0
	local to_charge_bots = 0
	for _, cell in pairs(network.cells) do
		charging_bots = charging_bots + cell.charging_robot_count
		to_charge_bots = to_charge_bots + cell.to_charge_robot_count
	end
	local cycle = global.cycle % (60 / global.update_ticks * global.avg_seconds)
	global.cycle = global.cycle + 1
	global.available_logi_bots[cycle] = available_logi_bots
	global.available_con_bots[cycle] = available_con_bots
	global.charging_bots[cycle] = charging_bots
	global.to_charge_bots[cycle] = to_charge_bots
	local available_logi_bots_avg = get_avg(global.available_logi_bots)
	local available_con_bots_avg = get_avg(global.available_con_bots)
	local charging_bots_avg = get_avg(global.charging_bots)
	local to_charge_bots_avg = get_avg(global.to_charge_bots)
	player.gui.left.bot_stat_gui.available_logi_bots_label.caption = string.format("Logistic: %u/%.0f of %u", network.all_logistic_robots - available_logi_bots, network.all_logistic_robots - available_logi_bots_avg, network.all_logistic_robots)
	player.gui.left.bot_stat_gui.available_con_bots_label.caption = string.format("Construction: %u/%.0f of %u", network.all_construction_robots - available_con_bots, network.all_construction_robots - available_con_bots_avg, network.all_construction_robots)
	player.gui.left.bot_stat_gui.charging_bots_label.caption = string.format("Charging: %u/%.0f", charging_bots, charging_bots_avg)
	player.gui.left.bot_stat_gui.to_charge_bots_label.caption = string.format("Waiting: %u/%.0f", to_charge_bots, to_charge_bots_avg)
end

local function on_tick()
	if game.tick % global.update_ticks == 0 then
		for _, player in pairs(game.players) do
			write_stats(player)
		end
	end
end

local function toggle_gui(player)
	if player.gui.left.bot_stat_gui then
		player.gui.left.bot_stat_gui.destroy()
		global.opening = true
		script.on_event(defines.events.on_tick, nil)
		return
	end
	script.on_event(defines.events.on_tick, on_tick)
    local bot_stat_gui = player.gui.left.add({type = "frame", name = "bot_stat_gui", direction = "vertical"})
	bot_stat_gui.add({type = "label", name = "available_logi_bots_label"})
	bot_stat_gui.add({type = "label", name = "available_con_bots_label"})
	bot_stat_gui.add({type = "label", name = "charging_bots_label"})
	bot_stat_gui.add({type = "label", name = "to_charge_bots_label"})
	write_stats(player)
end

local function open_gui(event)
    local player = game.players[event.player_index]
	toggle_gui(player)
end
script.on_event("open_gui", open_gui)